c $Header: /usr/people/flittner/radtran/flatpsv/src/Aerohump.f,v 1.14 1999/08/16 18:57:14 flittner Exp flittner $
******************************************************************
      subroutine Aerohump(Tfine,pavg,thet0i,thetj,phik,delphi,
     &dthjm,dthjp,forward,backward,fsbs,g,iflag)
c
c subroutine to calculate the average phase function over an interval
c centered on thetj, phik.
c***********************************************************************
c inputs:
c Tfine       = combinations of Herman's T1 & T2 functions
c thet0i      = input theta angle
c thetj       = output theta angle
c phik        = delta phi angle (phi in - phi out)
c delphi      = delta phi of averaged bin
c dthjm+dthjp = delta theta of averaged bin
c               The phase function of an average over the theta bin 
c ranging from thetj-dthjm to thej+dthjp.
c
c outputs:
c pavg        = 4 by 4 scattering matrix averaged over solid angle 
c               specified by the above angles.
c
C This subroutine calculates the average phase function for incident thet0i
C into scattered angle thetj and delta phi angle phik for the interval of
C delta theta and delta phi.  In the radiative transfer solution
C the integration is over delta theta and delta phi. Therefore the phase 
C function should represent this interval and an average value is required. 
C The solid angle is different for each theta0i, phi0i (the direction of the 
C incident ray), and also the scattering angle is a function of theta0,
C (phi0i-phij), therefore the integral of the phase function over solid angle 
C for thet0i thetj and phik divided by the solid angle, should be calculated for
C each thet0i,thetj and phik.
C NOTE:
C      1. Symmetry of the results (i.e. pavg will be the same for the upper
C         left hand and lower right hand, and will be the negative of for the
C         upper right hand lower left hand portions of the matrix.)
C         for phik =150, phik =210
C                  =120       =240
C                  =90        =270
C                  =60        =300
C                  =30        =330
C      2. For thet0i,thetj=thet0i,phik=0   pavg is the average phase 
C         function about the forward peak for incident ray at angle thet0i
C      3. For thet0i,thetj=180-thet0i,phik=180  pavg is the average phase
C         function about the backward direction for an incident ray at an angle
C         thet0i
C pavg= average of pf(i) for thet0i,thetj,phik  in delta theta=10
C         and delta phi=30. (i.e. between thetj+-5, and phik+-15)
C         pavg=(integral of pf(i)*d(solid angle))/integral of d(solid angle)
C
c---------------------------------------------------------------------
c note that minus signs are used in cos of theta angles because
c phase matrix is computed using a theta angle of 0.0 for a 
c beam traveling straight up and L.O.S. are read-in in the 
c subroutine read (which means that the beam traveling straight
c up is discribed by the LOS angle 180.0).
C      AMU = -cos(output angle)
C      ANU = sin(output angle)
C      AMUP = -cos(incident angle)
C      ANUP =  sin(incident angle)
C      AMUSQ = cos(inc)cos(inc)
C      AMUMU = cos(out)cos(inc)
C      AMUPS = cos(inc)cos(inc)
C      COOSN = cos(phi_output - phi_input)
C
c T1      = T1T1*
c T2      = T2T2*
c T3      = real(T1T2*)
c T4      = imag(T1T2*)
c
c last modified: Nov. 12, 1996
c purpose: input the switch for using Henyey-Greenstein phase function
c and the forward asymmetry parameter.
c
c last modified: July 22, 1997...def
c purpose: change switch for HG phase function to flag for size 
c          dist type (iflag).
c
c last modified: Dec. 30, 1997...def
c purpose: Increase second dimension of Tfine from 4 to 7 so that can
c          use phase functions created by tmatrix program and liou.
c
c last modified: Apr. 16, 1998...def
c purpose: Rotate matrix from paero2 from scattering plane to vertical plane
c
c***********************************************************************
      implicit none
      include 'consts.cmn'
      include 'param.inc'
c variables passed to Aerohump
      integer iflag
      real*8 thet0i,thetj,phik,delphi,pavg(4,4),Tfine(nan,7)
      real*8 dthjm,dthjp,g
      logical forward,backward,fsbs
c variables that are local
      integer idthet,idfi
      parameter (idthet=30,idfi=30)
      integer k,ii,jj,i,ndfi,ndthet,MT,MTP1
      real*8 ANUP,AMUP,COOSN,SIIN,COPSI,SISQ,CFISQ,SIFI,SFISQ
      real*8 AMUMU,AMUSQ,AMUPS,captheta,cos_captheta,onemya
      real*8 AMU,ANU,ang1,ang2,dmu,s,sumphi(4,4),T1,T2,T3,T4
      real*8 dt,th,delomg,costh0,sinth0,ph,phi
      real*8 c,theta(idthet),p(4,4)
      real*8 cosn(idfi),sinn(idfi),dphi,YA,tol
      real*8 costhj,thetmin,sumphiconst
      real*8 phg
      external phg
c
      if((iflag.eq.6).or.(iflag.eq.7))then
        do jj=1,4
           p(1,jj)=0.0d0
           p(2,jj)=0.0d0
           p(3,jj)=0.0d0
           p(4,jj)=0.0d0
        enddo
      endif
      tol=1.0e-04
      c=1.0d0
      s=1.0d0
c
      dphi=delphi/dble(idfi)
      dt=(dthjm+dthjp)/dble(idthet)
      ndfi=idfi
      ndthet=idthet
      costh0=dcos(thet0i*dr)
      sinth0=abs(sin(thet0i*dr))
      costhj=dcos(thetj*dr)
      if(forward)then
         c=2.0d0
         s=-1.0d0
         ndfi=ndfi/2
      else if(backward)then
         c=2.0d0
         s=1.0d0
         ndfi=ndfi/2
      endif
      sumphiconst=c*dphi*dr

      delomg=(dcos((thetj-dthjm)*dr)-dcos((thetj+dthjp)*dr))*
     &delphi*dr
c
      if(delomg.gt.0.0d0)then

C 2. CALCULATE PHI, SUCH THAT EACH DEGREE FROM -/+ DPHI/2
C FROM PHI IS ACCOUNTED FOR, THAT IS PHI=PHIK-14.5,-13.5,...,
C -0.5+PHIK,...14.5+PHIK FOR DELPHI OF 30 DEGREES

         do k=1,ndfi
            ph=dble(k)
            phi=phik-s*((delphi+dphi)*0.5d0-ph*dphi)
            cosn(k)=cos(phi*dr)
            sinn(k)=sin(phi*dr)
         enddo

C 1. SET THETA ARRAY THETA(I)=THETJ-DTHJM...,THETJ+DTHJP  
C DELTHE/IDTHET IS THE WIDTH OF THE FINE INTERVAL (THERE ARE IDTHET INTERVALS)
C IN THE RANGE AROUND THETJ SUCH THAT THERE ARE 10 ANGLES ABOUT THETA(I) 
C DTHETA=THETA(I+1)-THETA(I)=CONST=1.
         th=0.0d0
         thetmin=thetj-dthjm-dt*0.5d0
         do 20 i=1,ndthet
            th=th+1.0d0
            theta(i)=thetmin+dt*th
20       continue
c
         AMUP=-costh0
         ANUP=sinth0
         AMUPS=AMUP*AMUP
c initialize pavg to zero
         do jj=1,4
            pavg(1,jj)=0.0d0
            pavg(2,jj)=0.0d0
            pavg(3,jj)=0.0d0
            pavg(4,jj)=0.0d0
         enddo
c loop over output theta angle
         do 40 i=1,ndthet
            AMU=-cos(theta(i)*dr)
            ANU=abs(sin(theta(i)*dr))
            AMUSQ=AMU*AMU
            AMUMU=AMU*AMUP
c initialize sumphi to zero
            do jj=1,4
               sumphi(1,jj)=0.0d0
               sumphi(2,jj)=0.0d0
               sumphi(3,jj)=0.0d0
               sumphi(4,jj)=0.0d0
            enddo
c integrate over azimuthal angles
            do 30 k=1,ndfi
               COOSN=COSN(k)
               SIIN=SINN(k)
               COPSI=ANU*ANUP+AMU*AMUP*COOSN
               SISQ=COPSI*COPSI
               CFISQ=COOSN*COOSN
               SIFI=COPSI*COOSN
               SFISQ=SIIN*SIIN
               cos_captheta=AMUMU+ANUP*ANU*COOSN
               if(iflag.eq.6)then
                 do jj=1,4
                    p(1,jj)=0.0d0
                    p(2,jj)=0.0d0
                    p(3,jj)=0.0d0
                    p(4,jj)=0.0d0
                 enddo
                 p(1,1)=phg(cos_captheta,g)
               else if(iflag.eq.7)then
                 call paero2(Tfine,p,captheta,cos_captheta,fsbs,
     &ANUP,AMUP,AMU,ANU,COOSN,SIIN)
               else
                 call paero(Tfine,p,ANUP,AMUP,COOSN,SIIN,COPSI,SISQ,
     &CFISQ,SIFI,SFISQ,AMUMU,AMUSQ,AMUPS,captheta,cos_captheta,
     &AMU,ANU,fsbs)
               endif
               do jj=1,4
                  sumphi(1,jj)=p(1,jj)*sumphiconst+sumphi(1,jj)
                  sumphi(2,jj)=p(2,jj)*sumphiconst+sumphi(2,jj)
                  sumphi(3,jj)=p(3,jj)*sumphiconst+sumphi(3,jj)
                  sumphi(4,jj)=p(4,jj)*sumphiconst+sumphi(4,jj)
               enddo
30          continue

C CALCULATE THE INTEGRAL PF(I)*SIN(THETA)DTHETA
C SUMPHI IS THE INTEGRAL OF THE PHASE FUNCTION OVER PHI 
C COS(C1)-COS(C2)=INTEGRAL OF SIN(THETA)DTHETA FOR THE PHASE FUNCTION 
C CALCULATED FOR THETA(I),THET0I       
C INTEGRATE OVER THETA

            ang1=(theta(i)-dt/2.0d0)*dr
            ang2=(theta(i)+dt/2.0d0)*dr
            dmu=(cos(ang1)-cos(ang2))
            if(dmu.lt.0.0)then
               write(6,*)'hey what is the deal?'
               write(6,*)'domega (solid angle) is zero.'
               write(6,*)'dmu=',dmu
               write(6,*)'theta from',theta(i)-dt/2.0d0,' to',
     &theta(i)+dt/2.0d0
               write(6,*)'ang1=',ang1,'ang2=',ang2
               write(6,*)theta
               stop
            endif
            do ii=1,4
               do jj=1,4
                  pavg(ii,jj)=pavg(ii,jj)+sumphi(ii,jj)*dmu
               enddo
            enddo
40       continue

C SUMPHS IS THE INTEGRAL OF THE PHASE FUNCTION AROUND THATA0,PHI0 WITHIN
C DELTA PHI AND DELTA THETA

         do ii=1,4
            do jj=1,4
               pavg(ii,jj)=pavg(ii,jj)/delomg
            enddo
         enddo
      else
         write(6,*)'hey what is the deal?'
         write(6,*)'delomega (solid angle) is zero.'
         stop
      end if
      return
      end
***********************************************************************
