c $Header: /usr/people/flittner/radtran/flatpsv/src/Aerophase.f,v 1.14 1999/08/16 18:57:14 flittner Exp flittner $
************************************************************************
      subroutine Aerophase(costh0,sinth0,costh,sinth,cosn,sinn,nlos,
     &Tfine,wavel,QS,fmatrx,i,thet0i,thd,delphi,avgall,np,const,
     &g,iflag)
************************************************************************
c subroutine to calculate the Aerosol scattering phase matrix.
c If avgall is set to .TRUE. then the phase function will be 
c averaged for each output bin.  An output bin is specified
c by a set of theta and phi angles and some delta theta and 
c delta phi angles.  Otherwise, the phase function is computed
c only for the exact scattering angle.
c inputs:
c costh0 = -cos(incident theta angle)
c sinth0 = sin(incident theta angle)
c costh  = array of cos(output theta angles)
c sinth  = array of sin(output theta angles)
c cosn   = array of cos(delta phi angles)
c sinn   = array of sin(delta phi angles)
c nlos   = number of output theta angles
c Tfine  = Combinations of T1 & T2 functions found in Herman's dissertation.
c         To be used in phase matrix for scattering angles 0 to 180.
c         Tfine( ,1)=T1T1*
c         Tfine( ,2)=T2T2*
c         Tfine( ,3)=Real(T1T2*)
c         Tfine( ,4)=-Im(T1T2*)
c wavel  = wavelength in microns
c QS       = Scattering cross-section in square centimeters
c i      = theta input index for fmatrx
c thd       = output theta angles (assumed that array is in ascending order with
c         no elements equal to zero or 180.)
c delphi = delta phi of the output bins.  It is assumed that
c         it is the same for all output bins.  The same is 
c         not true for delta theta, which may be of uneven
c         spacing for the spherical atmosphere case.
c avgall = switch for averaging over the output bins.
c const = normalization constant =wavel*wavel/(4.0d0*pi*pi*QS*1.0d08)
c
c outputs:
c fmatrx = phase matrix that will be used in radiation program
c fmatrx(m,n,k,i,j) is m,n element of aerosol scattering matrix for input 
c angle i (acrcos(costh0)), output angles thd(j) and delta phi = (k-1)*delphi.
c
c last modified: Nov. 12, 1996
c purpose: input switch for henyey-greenstein phase function and 
c forward asymmetry parameter.
c
c last modified: July 22, 1997...def
c purpose: Change the switch for HG phase function to flag for 
c          size dist (iflag).
c
c last modified: Dec. 30, 1997...def
c purpose: Increase second dimension of Tfine from 4 to 7 so that can
c          use phase functions created by tmatrix program and liou.
c
c last modified: Apr. 16, 1998...def
c purpose: Rotate matrix from paero2 from scattering plane to vertical plane
c
      implicit none
      include 'consts.cmn'
      include 'param.inc'
c passed to Aerophase
      integer nlos,i,np,iflag
      real*8 costh(mlos),sinth(mlos),cosn(nfi),sinn(nfi)
      real*8 fmatrx(4,4,nfi,mlos,mlos),Tfine(nan,7),QS,wavel
      real*8 thd(mlos),thet0i,onemya,delphi,costh0,sinth0,const,g
      logical avgall
c local
      integer j,k,MT,MTP1
      real*8 ANUP,AMUP,COOSN,SIIN,COPSI,SISQ,CFISQ,SIFI,SFISQ
      real*8 AMUMU,AMUSQ,AMUPS,captheta,cos_captheta,AMU,ANU
      real*8 p(4,4),T1,T2,T3,T4,YA,thetj,phik,tol,dthjm,dthjp
      logical forward,backward,fsbs
       real*8 phg
       external phg
c
c
c note that minus signs are used in cos of theta angles because
c phase matrix is computed using a theta angle of 0.0 for a 
c beam traveling straight up and L.O.S. are read-in in the 
c subroutine read (which means that the beam traveling straight
c up is discribed by the LOS angle 180.0).
C      AMU = -cos(output angle)
C      ANU = sin(output angle)
C      AMUP = -cos(incident angle)
C      ANUP =  sin(incident angle)
C      AMUSQ = cos(inc)cos(inc)
C      AMUMU = cos(out)cos(inc)
C      AMUPS = cos(inc)cos(inc)
C      COOSN = cos(phi_output phi_input)
C
      tol=1.0e-06
c
      AMUP=costh0
      ANUP=sinth0
      AMUPS=AMUP*AMUP
c loop over output theta angle
      do 20 j=1,nlos
         AMU=-costh(j)
         ANU=sinth(j)
         AMUSQ=AMU*AMU
         AMUMU=AMU*AMUP
         thetj=thd(j)
         if(j.eq.1)then
            dthjm=thd(1)
         else
            dthjm=(thd(j)-thd(j-1))*0.5d0
         endif
         if(j.eq.nlos)then
            dthjp=180.0d0-thd(nlos)
         else
            dthjp=(thd(j+1)-thd(j))*0.5d0
         endif
C Loop over azimuth angle
         DO 10 k=1,nfi
            fsbs=.FALSE.
            forward=.FALSE.
            backward=.FALSE.
            if(k.eq.1)then
c k=1, phik=0.0
               phik=0.0d0
               if(abs(AMUP-AMU).lt.tol)then
                  forward=.TRUE.
                  backward=.FALSE.
                  fsbs=.TRUE.
               endif
            else if(k.eq.nfi)then
c k=nfi,phik=180.0
               phik=180.0d0
               if(abs(AMUP+AMU).lt.tol)then
                  forward=.FALSE.
                  backward=.TRUE.
                  fsbs=.TRUE.
               endif
            else
               phik=delphi*dble(k-1)
            endif
            if(avgall.or.forward.or.backward) then
               call Aerohump(Tfine,p,thet0i,thetj,phik,delphi,dthjm,
     &dthjp,forward,backward,fsbs,g,iflag)
            else
               COOSN=COSN(k)
               SIIN=SINN(k)
               COPSI=ANU*ANUP+AMU*AMUP*COOSN
               SISQ=COPSI*COPSI
               CFISQ=COOSN*COOSN
               SIFI=COPSI*COOSN
               SFISQ=SIIN*SIIN
               cos_captheta=AMUMU+ANUP*ANU*COOSN
               if(iflag.eq.6)then
                 p(1,1)=phg(cos_captheta,g)
               else if(iflag.eq.7)then
                 call paero2(Tfine,p,captheta,cos_captheta,fsbs,
     &ANUP,AMUP,AMU,ANU,COOSN,SIIN)
	       else
                 call paero(Tfine,p,ANUP,AMUP,COOSN,SIIN,COPSI,SISQ,
     &CFISQ,SIFI,SFISQ,AMUMU,AMUSQ,AMUPS,captheta,cos_captheta,
     &AMU,ANU,fsbs)
               endif
            endif
            if(np.eq.1)then
              if(iflag.eq.6)then
                fmatrx(1,1,k,i,j)=p(1,1)
              else
                fmatrx(1,1,k,i,j)=const*(p(1,1)+p(1,2)+p(2,1)+p(2,2))*
     &                                 0.5d0
              endif
            else
              if(iflag.eq.6)then
                fmatrx(1,1,k,i,j)=p(1,1)*0.5d0
                fmatrx(1,2,k,i,j)=p(1,1)*0.5d0
                fmatrx(2,1,k,i,j)=p(1,1)*0.5d0
                fmatrx(2,2,k,i,j)=p(1,1)*0.5d0
              else
                fmatrx(1,1,k,i,j)=const*p(1,1)
                fmatrx(1,2,k,i,j)=const*p(1,2)
                fmatrx(1,3,k,i,j)=const*p(1,3)
                fmatrx(1,4,k,i,j)=const*p(1,4)
                fmatrx(2,1,k,i,j)=const*p(2,1)
                fmatrx(2,2,k,i,j)=const*p(2,2)
                fmatrx(2,3,k,i,j)=const*p(2,3)
                fmatrx(2,4,k,i,j)=-fmatrx(1,4,k,i,j)
                fmatrx(3,1,k,i,j)=const*p(3,1)
                fmatrx(3,2,k,i,j)=const*p(3,2)
                fmatrx(3,3,k,i,j)=const*p(3,3)
                fmatrx(3,4,k,i,j)=const*p(3,4)
                fmatrx(4,1,k,i,j)=const*p(4,1)
                fmatrx(4,2,k,i,j)=-fmatrx(4,1,k,i,j)
                fmatrx(4,3,k,i,j)=const*p(4,3)
                fmatrx(4,4,k,i,j)=const*p(4,4)
              endif
            endif
10       CONTINUE
20    CONTINUE
      RETURN
      END
***********************************************************************
