c $Header: /usr/people/flittner/radtran/flatpsv/src/atmsubs.f,v 1.14 1999/08/16 18:57:13 flittner Exp flittner $
C***********************************************************************
C******************** SUBROUTINES USED IN FINDING **********************
C********************   ATMOSPHERIC STRUCTURE     **********************
C***********************************************************************

c***********************************************************************

	subroutine bhmie(x,refrel,nang,s1,s2,nan,HT1,HT2,qext,qscat,qback)

C Calculates amplitude scattering matrix elements and coeficients
C for extinction, total scattering and backscattering for a given size
C parameter and relative refractive index
C nstop is the maximum number of terms in the series
c T1 and T2 are Herman's T1 and T2 functions found in his disseration.
c HT1 & HT2 are these functions for the forward and backward directions.
c complex local

	implicit none
c
c local parameters
	integer ng,nd
c	parameter ( ng=91,nd=3000 )
	parameter ( ng=901,nd=3000 )
c integer passed
	integer nan,nang
c integer local
	integer n,k,jj,j,nstop,nmx
c real passed
	real*8 x,qext,qscat,qback
c real local
	real*8 apsi,psi,chi,apsi1,apsi0,fn,dn,t,psi0,psi1,rn,p,
     &		chi0,chi1,dang,ymod,dx,xstop,pie
	real*8 bi1,bi2
	real*8 piprime(3)
      	real*8 amu(ng),theta(ng),pi(ng),tau(ng),pi0(ng),pi1(ng)

c complex passed
      	complex*16 refrel,s1(nan),s2(nan),HT1(2),HT2(2)
c complex local 
      	complex*16 d(nd),y,xi,xi0,xi1,an,bn

c
	if(nang.gt.ng)then
		write(6,*)'will need to increase the number of ng to',
     &		nang,' in BHMIE.'
		stop
	endif
	pie=acos(-1.0d0)
        dx=x
        y=x*refrel
        xstop=x+4.0d0*x**0.3333d0+2.0d0
        nstop=xstop
        ymod=abs(y)
        nmx=max(xstop,ymod)+15
c
        dang=pie/(2.0d0*float(nang-1))

        do 10 j=1,nang
      		theta(j)=(float(j)-1.)*dang
        	amu(j)=cos(theta(j))
       		pi0(j)=0.0d0
        	pi1(j)=1.0d0
   10	continue
	piprime(1)=0.0d0
	piprime(2)=0.0d0
C LOGARITHMIC DERIVATIVE D(J) CALCULATED BY DOWNWARD RECURRENCE BEGINNING 
C WITH INITIAL VALUE 0.0 + I*0.0 AT J=NMX
        d(nmx)=cmplx(0.0,0.0)
        do 20 n=1,nmx-1
   		rn=nmx-n+1
        	d(nmx-n)=(rn/y)-(1./(d(nmx-n+1)+rn/y))
   20   continue
        do 40 j=1,2*nang-1
        	s1(j)=cmplx(0.0,0.0)
        	s2(j)=cmplx(0.0,0.0)
   40   continue
C RICCATI-BESSEL FUNCTION WITH REAL ARGUMENT X
C CALCULATED BY UPWARD RECURRENCE
        psi0=cos(dx)
        psi1=sin(dx)
        chi0=-sin(x)
        chi1=cos(x)
        apsi0=psi0
        apsi1=psi1
        xi0=cmplx(apsi0,-chi0)
        xi1=cmplx(apsi1,-chi1)
        qscat=0.0d0
        n=1
200     dn=n
        rn=n
        fn=(2.0d0*rn+1.0d0)/(rn*(rn+1.0d0))
        psi=(2.0d0*dn-1.0d0)*psi1/dx-psi0
        apsi=psi
        chi=(2.0d0*rn-1.)*chi1/x-chi0
        xi=cmplx(apsi,-chi)
        an=(d(n)/refrel+rn/x)*apsi-apsi1
        an=an/((d(n)/refrel+rn/x)*xi-xi1)
        bn=(refrel*d(n)+rn/x)*apsi-apsi1
        bn=bn/((refrel*d(n)+rn/x)*xi-xi1)
        qscat=qscat+(2.0d0*rn+1.0d0)*(abs(an)*abs(an)+abs(bn)*abs(bn))
        k=(n/2)*2
	if(k.eq.n) then
		t=1.0d0
		p=-1.0d0
	else
		t=-1.0d0
		p=1.0d0
	endif
        do 50 j=1,nang
   		jj=2*nang-j
        	pi(j)=pi1(j)
        	tau(j)=rn*amu(j)*pi(j)-(rn+1.)*pi0(j)
        	s1(j)=s1(j)+fn*(an*pi(j)+bn*tau(j))
        	s2(j)=s2(j)+fn*(an*tau(j)+bn*pi(j))
        	if(j.eq.jj)go to 50
        	s1(jj)=s1(jj)+fn*(an*pi(j)*p+bn*tau(j)*t)
        	s2(jj)=s2(jj)+fn*(an*tau(j)*t+bn*pi(j)*p)
   50	continue
	if(n.eq.1)then
		HT1(1)=-fn*pi(1)*bn
		HT2(1)=-fn*pi(1)*an
		HT1(2)=-fn*pi(1)*bn
		HT2(2)=-fn*pi(1)*an
	else
		HT1(1)=HT1(1)+fn*(piprime(3)*(an-bn)-pi(1)*bn)
		HT2(1)=HT2(1)+fn*(piprime(3)*(bn-an)-pi(1)*an)
		HT1(2)=HT1(2)+fn*(t*piprime(3)*(an+bn)-p*pi(1)*bn)
		HT2(2)=HT2(2)+fn*(t*piprime(3)*(bn+an)-p*pi(1)*an)
	endif
        psi0=psi1
        psi1=psi
        apsi1=psi1
        chi0=chi1
        chi1=chi
        xi1=cmplx(apsi1,-chi1)
        n=n+1
        rn=n*1.0d0
	bi1=(2.0d0*rn-1.0d0)/(rn-1.0d0)
	bi2=rn/(rn-1.0d0)
        do 60 j=1,nang
        	pi1(j)=bi1*amu(j)*pi(j)
        	pi1(j)=pi1(j)-bi2*pi0(j)
        	pi0(j)=pi(j)
   60   continue
	piprime(3)=(2.0d0*rn-1.)*pi0(1)+piprime(1)
	piprime(1)=piprime(2)
	piprime(2)=piprime(3)
        if(n-1-nstop)200,300,300
300     qscat=(2.0d0/(x*x))*qscat
        qext=(4.0d0/(x*x))*(dreal(s1(1)))
        qback=(4.0d0/(x*x))*abs(s1(2*nang-1))*abs(s1(2*nang-1))
        return
        end 

c***********************************************************************
       subroutine normalize(xmax,x,NNm1,oldmax)
       implicit none
       integer NNm1,i
       real*8 xmax,x(NNm1),oldmax,ratio
c
       ratio = xmax/oldmax
       do 10 i=1,NNm1,1
               x(i)=ratio*x(i)
10     continue
       return
       end 
**********************************************************************
       subroutine setzeros(x,n)
c
       integer n
       real*8 x(n)
c
       do 10 i=1,n
               x(i)=0.0d0
10     continue
       return
       end 
**********************************************************************
       subroutine rdin(z,con,N,num,fname,ierr)
c
c this subroutine will open constituent file 'fname' and read til the
c end of the file.
c
       implicit none
       integer N,ierr,num
       real*8 z(N),con(N)
       real*8 t1,t2
       character fname*(*)
       integer answer
       external answer
c
       ierr=0
5      open(unit=16,file=fname,status='old',err=2000)
       num=0
10     read(16,*,end=20,err=3000)t1,t2
         num=num+1
         if(num.gt.N)then
           write(6,*)'WHILE READING CONSTITUITE CONCENTRATION FILE '
           write(6,*)fname
           write(6,*)'HAVE TRIED TO READ MORE DATA THAN HAVE ROOM ',
     &'IN ARRAYS.'
           write(6,*)'WILL NEED TO INCREASE THE DEFAULT DIMENSION ',
     &'SIZE OF INPUT STRUCTURE DATA.'
           ierr=3
           return
         endif
         z(num)=t1
         con(num)=t2
         goto 10
20     close(16)
       return

2000   print*,'Error openning file ',fname,' to read for input of ',
     * 'constituite concentration.'
c       print*,'Do you wish to try opening the file again? (y/n)',
c     * ' (a no answer will cause termination of atmstr)'
c       read(5,'(a)')ans
c       if(answer(ans))30,2000,5
30     ierr=1
       return
3000   write(6,*)'Error reading file ',fname
       ierr=2
       return
       end
*************************************************************************
       integer function answer(text)
       character text
       if((text.eq.'y').or.(text.eq.'Y'))then
               answer=1
               return
       else if((text.eq.'n').or.(text.eq.'N'))then
               answer=-1
               return
       else
               answer=0
       end if
       return
       end
c***********************************************************************
        subroutine cndl(nlayr,ndl)
c routine to calc ndl so that ndl and nlayr are integer values for the 
c relationship, nlayr=160/ndl.
        implicit none
        integer nlayr,nnlay,ndl,nndl,nnnlay
        ndl=160/nlayr
	do while(mod(160,ndl).ne.0)
	   ndl=ndl-1
        enddo
        nndl=ndl+1
	do while(mod(160,nndl).ne.0)
	   nndl=nndl+1
        enddo
	nnlay=160/ndl
        nnnlay=160/(nndl)
        if(abs(nlayr-nnlay).gt.abs(nnnlay-nlayr))then
          ndl=nndl
        endif
        nlayr=160/ndl
	return
        end

C########################################################################
