c $Header: /usr/people/flittner/radtran/flatpsv/src/rdmodtrn.f,v 1.14 1999/08/16 18:57:13 flittner Exp flittner $
c***********************************************************************
      subroutine rdmodtrnatm(imodt,sfcprs,top,prtf,
     &NN,npr,naer,ntemp,noz,nh2o,ncld,zpres,ztemp,zaer,zoz,zwat,zc,
     &pres,temp,oz,aer,cld,wat,
     &y2,intpres,totoz,totwat,ttc,ttf,ttr,tto3,tth2o,oneatm)
c
c read modtran 35 layer atmosphere
c
c input:
c imodt - unit # of the modtran file
c sfcprs - pressure of the reflecting surface [mb]
c top - bottom of the chapman layer [km]
c prtf - logical switch to print files of the profiles
c 
c output:
c NN - max dimension of input profile arrays
c npr - dimension of the input profile =naer=ntemp=noz=nh2o=ncld
c zpres(npr) - alt. of input pressure profile =zaer=ztemp=zoz=zwat=zc [km]
c pres(npr) - input pressure [mb]
c temp(ntemp) - input temp. [k]
c oz(noz) -
c aer(naer) -
c cld(ncld) -
c wat(nwat) -
c y2(NN) - work array
c intpres - pressure at the levles zstd
c totoz -
c totwat -
c ttr - total rayleigh od
c ttf - total aerosol od
c tto3 - total ozone od
c tth2o - total water vapor od
c ttc - total cloud od
c zsfc - alt. of the reflectinc surface [km]
c totoz
c totwat
c oneatm - pressure at the bottom of the input profile
c
c transferred via the common blocks
c zstd - standard alt. grid [km]
c nstd - # of levels in std. grid 
c
c internal
c prestop - pressure at the bottom of the chapman layer
c zmax -
c zmin -
C-----------------------------------------------------------------------
       implicit none

       include 'consts.cmn'
       include 'catmdata.cmn'
       include 'cstdatm.cmn'
c
c passed in
       integer imodt
       real*8 sfcprs,top
       logical prtf
c passed out
       integer NN
       integer npr,naer,noz,nh2o,ntemp,ncld
       real*8 zpres(NN),pres(NN),zaer(NN),aer(NN),zoz(NN),oz(NN)
       real*8 zwat(NN),wat(NN),ztemp(NN),temp(NN),y2(NN)
       real*8 zc(NN),cld(NN),intpres(NN)
       real*8 ttf,ttr,tto3,tth2o,ttc,zsfc,totoz,totwat,oneatm
c local
       integer nmodtrn,i,j,ilay,mode
       parameter (nmodtrn=36)
       real*8 dtaus(nmodtrn,9),v,yp1,ypn,prestop,zmin,zmax
       real*8 dtauc(nmodtrn-1),ssalb(nmodtrn-1)
       character buf*1
c**************************************************************
c
c read the modtran atmosphere file
c
       read(imodt,'(a)')buf 
       do i=1,nmodtrn-1
          read(imodt,*)ilay,(dtaus(i,j),j=1,9)
          dtaus(i,8)=dtaus(i,9)-dtaus(i,8)
          if(prtf)write(6,*)ilay,dtaus(i,9)
       enddo
c
c read the other stuff (zpres,pres,temp)
c
       read(imodt,'(5x,i4)')npr
       if(prtf)write(6,*)'rdmodtrn. npr ',npr
       read(imodt,'(/,7(5e12.5,/),e12.5)')(zpres(i),i=1,npr)
       if(prtf)write(6,'("z",/,7(5e12.5,/),e12.5)')(zpres(i),i=1,npr)
       read(imodt,'(/,7(5e12.5,/),e12.5)')(pres(i),i=1,npr)
       if(prtf)write(6,'("p",/,7(5e12.5,/),e12.5)')(pres(i),i=1,npr)
       read(imodt,'(/,7(5e12.5,/),e12.5)')(temp(i),i=1,npr)
       if(prtf)write(6,'("t",/,7(5e12.5,/),e12.5)')(temp(i),i=1,npr)
c
c now skip the rest of the stuff
c
c check to atmo passed to disort to see if the same tau_total
c in each layer and the same w0
c
      do i=1,11
         read(imodt,'(a)')buf
      enddo
      read(imodt,*)dtauc
      read(imodt,'(a)')buf
      read(imodt,*)ssalb
      do i=1,26
         read(imodt,'(a)')buf
      enddo
c check the dtau
       do i=1,nmodtrn-1
          if(abs(1.-dtauc(nmodtrn-i)/(dtaus(i,8)+dtaus(i,7)))
     &.gt.1.0e-04)then
            write(6,2040)'rdmod. dtau does not match',zpres(i),
     &dtauc(nmodtrn-i)/(dtaus(i,8)+dtaus(i,7)),
     &ssalb(nmodtrn-i)/(dtaus(i,7)/(dtaus(i,7)+dtaus(i,8)))
          endif
       enddo
c was 62
       do i=1,npr
          zaer(i)=zpres(i)
          zoz(i)=zpres(i)
          zwat(i)=zpres(i)
          ztemp(i)=zpres(i)
       enddo
c interpolate pressure to the zstd points
c
c will use "natrual" spline
       yp1=1.0d30
       ypn=1.0d30
       mode=2
c what is the pressure at the top of this profile (@ z=top)?
       call spline(zpres,pres,npr,yp1,ypn,y2,mode)
       call splint(zpres,pres,y2,npr,top,prestop,mode)
       write(6,2000)prestop
c what is the pressure at the bottom of this profile (@ z=0)?
       write(6,2010)zpres(1),pres(1)
c will normalize pressure profile to bottom of the input profile,
c so will set oneatm equal to pres(1)
       oneatm=pres(1)
c what is the alt. of the reflecting surface?
       if(abs(pres(1)-sfcprs).gt.smallest)then
         mode=1
         call spline(pres,zpres,npr,yp1,ypn,y2,mode)
         call splint(pres,zpres,y2,npr,sfcprs,zsfc,mode)
       else
         zsfc=zpres(1)
       endif
       write(6,2020)zsfc,sfcprs
c
       zmax=zpres(npr)
       zmin=zpres(1)
c readjust the top of the zstd array
       i=1
       do while((zstd(i).lt.zmax).and.(i.lt.N))
          i=i+1
       enddo
       nstd=i
c adjust the bottom of the zstd array match the input files
       if(zstd(1).lt.zmin)then
         zstd(1)=zmin
         i=2
         do while(zstd(i).lt.zstd(1))
            i=i+1
         enddo
         if(i.gt.2)then
           do j=i,nstd
              zstd(2+(j-i))=zstd(j)
           enddo
           nstd=nstd-(i-2)
         endif
       endif
       mode=2
       call spline(zpres,pres,npr,yp1,ypn,y2,mode)
       do i=1,nstd
          call splint(zpres,pres,y2,npr,zstd(i),intpres(i),mode)
       enddo
c scale height for alt. above zstd(nstd)
       Hp=(dzstd(nstd-1))/log(intpres(nstd-1)/intpres(nstd))
       write(6,'(a31,1pE13.4,a6)')'divide. Pressure scale height=',
     &Hp,' (km).'
       if(prtf)then
         open(92,file='intpres.dat',status='unknown')
         do i=1,nstd
            write(92,*)zstd(i),intpres(i)
         enddo
         close(92)
       endif
c Now interpolate profiles to find values at zstd.
       do i=1,nstd
          ttotal(i)=0.0d0
       enddo
c
c now construct the total vertical optical thickness arrays
c
      ntemp=npr
      noz=npr
      naer=npr
      nh2o=npr
c
c rayleigh
c
      j=7
      dtaus(npr,j)=pres(npr)*dtaus(npr-1,j)/(pres(npr-1)-pres(npr))
      do i=npr-1,1,-1
         dtaus(i,j)=dtaus(i+1,j)+dtaus(i,j)
      enddo
      mode=2
      call spline(zpres,dtaus(1,j),npr,yp1,ypn,y2,mode)
      do i=1,nstd
         call splint(zpres,dtaus(1,j),y2,npr,zstd(i),tauray(i),mode)
         ttotal(i)=tauray(i)
      enddo
      ttr=tauray(1)
      if(ttr.gt.0.0)then
        rayleigh=.TRUE.
      else
        rayleigh=.FALSE.
      endif
c
c aerosol
c
      j=3
      dtaus(npr,j)=0.0
      do i=npr-1,1,-1
         dtaus(i,j)=dtaus(i+1,j)+dtaus(i,j)
      enddo
      mode=2
      call spline(zpres,dtaus(1,j),npr,yp1,ypn,y2,mode)
      do i=1,nstd
         call splint(zpres,dtaus(1,j),y2,npr,zstd(i),tauf(i),mode)
         ttotal(i)=ttotal(i)+tauf(i)
      enddo
      ttf=tauf(1)
      if(ttf.gt.0.0)then
        aerosol=.TRUE.
      else
        aerosol=.FALSE.
      endif
c
c cloud
c
      j=5
      dtaus(npr,j)=0.0
      do i=npr-1,1,-1
         dtaus(i,j)=dtaus(i+1,j)+dtaus(i,j)
      enddo
      mode=2
      call spline(zpres,dtaus(1,j),npr,yp1,ypn,y2,mode)
      do i=1,nstd
         call splint(zpres,dtaus(1,j),y2,npr,zstd(i),tauc(i),mode)
         ttotal(i)=ttotal(i)+tauc(i)
      enddo
      ttc=tauc(1)
      if(ttc.gt.0.0)then
        cloud=.TRUE.
      else
        cloud=.FALSE.
      endif
c
c ozone (all gases absorbers)
c
      j=8
      dtaus(npr,j)=0.0
      do i=npr-1,1,-1
         dtaus(i,j)=dtaus(i+1,j)+dtaus(i,j)
      enddo
      mode=2
      call spline(zpres,dtaus(1,j),npr,yp1,ypn,y2,mode)
      do i=1,nstd
         call splint(zpres,dtaus(1,j),y2,npr,zstd(i),tauo3(i),mode)
         ttotal(i)=ttotal(i)+tauo3(i)
      enddo
      tto3=tauo3(1)
      if(tto3.gt.0.0)then
        ozone=.TRUE.
      else
        ozone=.FALSE.
      endif
c
c zero-out the h2o
c
      do i=1,nstd
         tauh2o(i)=0.0
      enddo
      water=.FALSE.
      return
2000   format(' divide. The pres. at the top of the input profile is',
     &1PE12.4,' (mb)')
2010   format(' divide. The pres. at the bottom of the input profile ',
     &F5.2,' (km) is ',f7.2,' (mb)')
2020   format(' divide. The pres. at the reflecting surface ',
     &F6.2,' (km) is ',f7.2,' (mb)')
2040   format(a26,f6.3,2f8.4)
      end
